/*******************************************************************************
 * Copyright (c) 2004, 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 4, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *    Justin Braga - modified start method to register ValidClassActionFilter
 *    Liam Morley - changed adapter factory to use IType and ICompilationUnit
 * 					instead of IResource, added call for unregistering adapter
 * 					factory to the stop() method 
 * 
 * $Id: ObjBenchPlugin.java,v 1.1 2006/01/23 16:05:52 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.core.runtime.IAdapterManager;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IType;
import org.eclipse.ui.IActionFilter;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import edu.wpi.ebob.actions.ValidClassActionFilter;
import edu.wpi.ebob.internal.jem.JEMClassManager;
import edu.wpi.ebob.model.ObjectBench;
import edu.wpi.ebob.model.cm.IClassManager;

/**
 * The main plugin class to be used in the desktop.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class ObjBenchPlugin extends AbstractUIPlugin {
    //The shared instance.
    private static ObjBenchPlugin plugin;

    /**
     * Returns the shared instance.
     */
    public static ObjBenchPlugin getDefault() {
        return plugin;
    }

    /**
     * Returns the string from the plugin's resource bundle, or 'key' if not
     * found.
     */
    public static String getResourceString(String key) {
        ResourceBundle bundle = ObjBenchPlugin.getDefault().getResourceBundle();
        try {
            return (bundle != null) ? bundle.getString(key) : key;
        } catch (MissingResourceException mre) {
            return key;
        }
    }

    // Used to register the "isValidClass" action filter
    private IAdapterFactory actionFilterAdapterFactory;

    // the container for the model objects
    private final ObjectBench bench;

    // Manages user-created classes.
    private IClassManager classManager;

    //Resource bundle.
    private ResourceBundle resourceBundle;

    /**
     * The constructor.
     */
    public ObjBenchPlugin() {
        super();
        plugin = this;
        bench = new ObjectBench();
        classManager = new JEMClassManager();
        try {
            resourceBundle = ResourceBundle.getBundle("edu.wpi.ebob.messages"); //$NON-NLS-1$
        } catch (MissingResourceException mre) {
            resourceBundle = null;
        }
    }

    /**
     * @return Returns the object bench associated with this plug-in.
     */
    public ObjectBench getBench() {
        return bench;
    }

    /**
     * @return the class manager for retrieving references to classes and
     *         objects created in the JVM.
     */
    public IClassManager getClassManager() {
        return classManager;
    }

    /**
     * Returns the plugin's resource bundle,
     */
    public ResourceBundle getResourceBundle() {
        return resourceBundle;
    }

    /**
     * This method is called upon plug-in activation
     */
    public void start(BundleContext context) throws Exception {
        super.start(context);

        // register adapter factory for context-click
        IAdapterManager manager = Platform.getAdapterManager();
        actionFilterAdapterFactory = new IAdapterFactory() {
            /**
             * @see IAdapterFactory#getAdapter(Object, Class)
             */
            public Object getAdapter(Object adaptableObject, Class adapterType) {
                // we only know how to deal with IActionFilter and
                // IType/ICompilationUnit
                if (adapterType == IActionFilter.class
                        && (adaptableObject instanceof ICompilationUnit)
                        || (adaptableObject instanceof IType))
                    return new ValidClassActionFilter();
                // we should never get here
                throw new AssertionError();
            }

            /**
             * @see IAdapterFactory#getAdapterList()
             */
            public Class[] getAdapterList() {
                return new Class[] { IActionFilter.class };
            }

        };
        manager.registerAdapters(actionFilterAdapterFactory, IType.class);
        manager.registerAdapters(actionFilterAdapterFactory,
                ICompilationUnit.class);
    }

    /**
     * This method is called when the plug-in is stopped
     */
    public void stop(BundleContext context) throws Exception {
        try {
            IAdapterManager manager = Platform.getAdapterManager();
            manager.unregisterAdapters(actionFilterAdapterFactory);
            classManager.dispose();
            classManager = null;
        } finally {
            super.stop(context);
        }
    }
}