/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 10, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: CreateInstanceAction.java,v 1.1 2006/01/23 16:07:15 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.actions;

import org.eclipse.jdt.core.IType;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.wizards.AddToObjectBenchWizard;

/**
 * An action for adding instances to the Object Bench.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class CreateInstanceAction extends Action {

    private static final ImageDescriptor enabledImg = PlatformUI.getWorkbench()
            .getSharedImages().getImageDescriptor(
                    ISharedImages.IMG_TOOL_NEW_WIZARD);

    private static final ImageDescriptor disabledImg = PlatformUI
    .getWorkbench().getSharedImages().getImageDescriptor(
            ISharedImages.IMG_TOOL_NEW_WIZARD_DISABLED);

    private final IType type;
    private final Shell shell;

    /**
     * Creates a new action, tied to the given shell.
     * 
     * @param shell
     *            The shell that will own the wizard.
     * @see AddToObjectBenchWizard
     */
    public CreateInstanceAction(final Shell shell) {
        this(shell, null);
    }

    /**
     * Creates a new action, tied to the given shell. Starts the wizard with the
     * given class.
     * 
     * @param shell
     *            The shell that will own the wizard.
     * @param type
     *            The class to begin the wizard with.
     * @see AddToObjectBenchWizard
     */
    public CreateInstanceAction(final Shell shell, final IType type) {
        this.shell = shell;
        this.type = type;
        setText(ObjBenchPlugin.getResourceString("CreateInstanceAction.text")); //$NON-NLS-1$
        setToolTipText(ObjBenchPlugin.getResourceString("CreateInstanceAction.tooltip")); //$NON-NLS-1$
        setImageDescriptor(enabledImg);
        setDisabledImageDescriptor(disabledImg);
    }

    /**
     * Runs the wizard for adding objects to the bench.
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        AddToObjectBenchWizard wizard = new AddToObjectBenchWizard();
        if (type != null)
            wizard.setType(type);
        WizardDialog dialog = new WizardDialog(shell, wizard);
        dialog.create();
        dialog.open();
    }
}