/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 22, 2004
 *
 * Contributors:
 *    Justin Braga - initial API and implementation
 *    Liam Morley  - changed interface implementation to a subinterface, added
 *                   setActivePart and implemented methods (11/24/04)
 * 
 * $Id: ExecuteAddWizardActionDelegate.java,v 1.1 2006/01/23 16:07:15 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.actions;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import edu.wpi.ebob.wizards.AddToObjectBenchWizard;

/**
 * Invokes the {@link AddToObjectBenchWizard}on a selection in the workbench.
 * 
 * @author <a href="mailto:trevelyan2001@users.sourceforge.net">Justin Braga
 *         </a>
 * @see AddToObjectBenchWizard
 */
public class ExecuteAddWizardActionDelegate implements IObjectActionDelegate {

    private ISelection selection;

    private IWorkbenchPart part;

    /**
     * @see IActionDelegate#run(IAction)
     */
    public void run(IAction action) {
        AddToObjectBenchWizard wizard = new AddToObjectBenchWizard();
        IStructuredSelection structuredSelection = (IStructuredSelection) selection;
        wizard.init(part.getSite().getWorkbenchWindow().getWorkbench(),
                structuredSelection);

        // Instantiates the wizard container with the wizard and opens it
        WizardDialog dialog = new WizardDialog(part.getSite().getShell(),
                wizard);
        dialog.create();
        dialog.open();
    }

    /**
     * @see IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        this.selection = selection;
    }

    /**
     * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        part = targetPart;
    }
}