/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Mar 26, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: DetailsDialogUtil.java,v 1.1 2006/01/23 16:08:04 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui.dialogs;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.text.MessageFormat;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Shell;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;

/**
 * Convenience class for providing a stack trace dialog with optional details.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class DetailsDialogUtil {

    /**
     * Retrieves a {@link MultiStatus}made out of multiple Status objects, each
     * Status object representing a line in the stacktrace.
     */
    private static IStatus getStatus(Throwable t) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        PrintStream ps = new PrintStream(baos);
        t.printStackTrace(ps);

        try {
            baos.flush();
        } catch (IOException ioe) {
            // what are you going to do?
            ObjBenchLogger.logError(ioe);
        }
        ps.close();

        String trace = baos.toString();
        String[] lines = trace.split("\n"); //$NON-NLS-1$

        IStatus[] stati = new IStatus[lines.length];
        if (lines.length > 0) {
            stati[0] = new Status(IStatus.ERROR, ObjBenchPlugin.getDefault()
                    .getBundle().getSymbolicName(), IStatus.ERROR, lines[0]
                    .trim(), t);
            for (int i = 1; i < stati.length; i++)
                stati[i] = new Status(IStatus.ERROR, ObjBenchPlugin
                        .getDefault().getBundle().getSymbolicName(),
                        IStatus.ERROR, "        " + lines[i].trim(), t); //$NON-NLS-1$
        }
        String message = t.getLocalizedMessage();
        if (null == message)
            message = MessageFormat.format(ObjBenchPlugin.getResourceString("DetailsDialogUtil.errorHasOccurred"), //$NON-NLS-1$
                    new Object[] { t });
        return new MultiStatus(ObjBenchPlugin.getDefault().getBundle()
                .getSymbolicName(), IStatus.ERROR, stati, message, t);
    }

    /**
     * Opens an error dialog with details concerning the stack trace of the
     * given exception.
     * 
     * @param parent
     *            the parent shell
     * @param t
     *            the exception to display
     * @return the return code of a {@link org.eclipse.jface.dialogs.Dialog}.
     */
    public static int openError(Shell parent, Throwable t) {
        return ErrorDialog.openError(parent, null, null, getStatus(t));
    }
}