/*******************************************************************************
 * Copyright (c) 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Mar 17, 2005
 *
 * Contributors:
 *    Justin Braga - initial API and implementation
 * 
 * $Id: MethodResultDialog.java,v 1.1 2006/01/23 16:08:04 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui.dialogs;

import java.text.MessageFormat;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.utils.ObjectTreeViewer;
import edu.wpi.ebob.internal.inspect.RootNode;
import edu.wpi.ebob.model.cm.IMethod;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * @author <a href="mailto:trevelyan2001@users.sourceforge.net">Justin Braga</a>
 *
 */
public class MethodResultDialog extends Dialog {

	private TreeViewer treeViewer;
	private IObject result;
	private IMethod method;

	/**
	 * @param parentShell
	 */
	public MethodResultDialog(Shell parentShell, IMethod method, IObject result) {
		super(parentShell);
		this.method = method;
		this.result = result;
	}
	
	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
	}
	
	/**
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) {
	       Composite composite = (Composite) super.createDialogArea(parent);
	        
	        // The Info Text
	        Label info = new Label(composite, SWT.WRAP);
	        try {
                info.setText(MessageFormat.format(ObjBenchPlugin
                        .getResourceString("MethodResultDialog.info"), //$NON-NLS-1$
                        new Object[] { method.getName() }));
            } catch (RemoteException e) {
                ObjBenchLogger.logError(e);
            }
	        GridData data = new GridData(GridData.GRAB_HORIZONTAL
	                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
	                | GridData.VERTICAL_ALIGN_CENTER);
	        data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
	        info.setLayoutData(data);
	        info.setFont(parent.getFont());
	        
	        // The object tree
	        ObjectTreeViewer otv = new ObjectTreeViewer(composite);
	        treeViewer = otv.getTreeViewer();
	        RootNode rn = new RootNode(method.getReturnType(), result);
	        data = new GridData(GridData.FILL_HORIZONTAL);
	        data.heightHint = 200;
	        treeViewer.getTree().setLayoutData(data);
	        treeViewer.setInput(rn);
	        treeViewer.expandToLevel(2);
	        
			return composite;
	}
	
	/**
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(ObjBenchPlugin
                .getResourceString("ExecuteMethodDialog.title")); //$NON-NLS-1$
	}
}
