/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 4, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: RemoveInstanceDialog.java,v 1.1 2006/01/23 16:08:04 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IconAndMessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.Instance;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class RemoveInstanceDialog extends IconAndMessageDialog {
    /**
     * Return code constant (value 2) indicating the instance should be deleted
     * by force.
     */
    public static final int DELETE_FORCE = 2;

    /**
     * Return code constant (value 2) indicating the instance should be deleted
     * along with all children.
     */
    public static final int DELETE_CASCADE = 3;

    private Instance instance;

    private int returnCode;

    /**
     * @param parentShell
     */
    public RemoveInstanceDialog(Shell parentShell, Instance instance) {
        super(parentShell);
        returnCode = DELETE_FORCE;
        this.instance = instance;
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        createMessageArea(parent);
        Composite composite = (Composite) super.createDialogArea(parent);

        // question
        Label question = new Label(composite, SWT.WRAP);
        question.setText(ObjBenchPlugin
                .getResourceString("RemoveInstanceDialog.questionLabel")); //$NON-NLS-1$
        GridData data = new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER);
        data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
        question.setLayoutData(data);
        question.setFont(parent.getFont());

        // delete force
        Button deleteForce = new Button(composite, SWT.RADIO | SWT.LEFT);
        deleteForce.setText(ObjBenchPlugin
                .getResourceString("RemoveInstanceDialog.force")); //$NON-NLS-1$
        deleteForce.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                returnCode = DELETE_FORCE;
            }

            public void widgetDefaultSelected(SelectionEvent e) {
                returnCode = DELETE_FORCE;
            }
        });

        // delete cascade
        Button deleteCascade = new Button(composite, SWT.RADIO | SWT.LEFT);
        deleteCascade.setText(ObjBenchPlugin
                .getResourceString("RemoveInstanceDialog.cascade")); //$NON-NLS-1$
        deleteCascade.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                returnCode = DELETE_CASCADE;
            }

            public void widgetDefaultSelected(SelectionEvent e) {
                returnCode = DELETE_CASCADE;
            }
        });
        return composite;
    }

    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(ObjBenchPlugin
                .getResourceString("RemoveInstanceDialog.title")); //$NON-NLS-1$
    }

    /**
     * Returns either <code>DELETE_FORCE</code>,<code>DELETE_CASCADE</code>,
     * or <code>CANCEL</code>.
     * 
     * @see org.eclipse.jface.window.Window#getReturnCode()
     */
    public int getReturnCode() {
        int choice = super.getReturnCode();
        return (choice == OK) ? returnCode : choice;
    }

    /**
     * @see org.eclipse.jface.dialogs.IconAndMessageDialog#getImage()
     */
    protected Image getImage() {
        return getQuestionImage();
    }

    /**
     * Returns the RemoveInstanceDialog return code instead of just
     * <code>OK</code> or <code>CANCEL</code>.
     * 
     * @see org.eclipse.jface.window.Window#open()
     * @see #getReturnCode()
     */
    public int open() {
        super.open();
        return getReturnCode();
    }
}