/*******************************************************************************
 * Copyright (c) 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Mar 2, 2005
 *
 * Contributors:
 *    Justin Braga - initial API and implementation
 * 
 * $Id: ParameterTableViewer.java,v 1.1 2006/01/23 16:06:36 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.gui.utils;

import org.eclipse.jface.util.ListenerList;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import edu.wpi.ebob.internal.wizards.ArgsCellModifier;
import edu.wpi.ebob.internal.wizards.ArgsContentProvider;
import edu.wpi.ebob.internal.wizards.ArgsLabelProvider;
import edu.wpi.ebob.internal.wizards.IArgsConstants;
import edu.wpi.ebob.model.Parameter;
import edu.wpi.ebob.model.cm.IClass;

/**
 * @author <a href="mailto:trevelyan2001@users.sourceforge.net">Justin Braga</a>
 *
 */
public class ParameterTableViewer {
	
	private ListenerList listeners = new ListenerList(2);
	private Table table;
	private TableViewer tableViewer;
	private static final int TABLE_STYLE = SWT.SINGLE | 
			SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
    		| SWT.FULL_SELECTION | SWT.HIDE_SELECTION;
	
	public ParameterTableViewer(Composite parent) {
		createTable(parent);
		createTableViewer();
	}
	
	private void createTable(Composite parent) {
		table = new Table(parent, TABLE_STYLE);
	      table.setLayoutData(new GridData(GridData.FILL_BOTH));
	        table.setLinesVisible(true);
	        table.setHeaderVisible(true);

	        new TableColumn(table, SWT.LEFT, 0)
	        	.setText(IArgsConstants.TABLE_COL_TYPE);
	        new TableColumn(table, SWT.LEFT, 1)
	        	.setText(IArgsConstants.TABLE_COL_ARGUMENT);

	        // adjust the column sizes the first time this is displayed
	        table.addPaintListener(new PaintListener() {
	        	public void paintControl(PaintEvent e) {
	        		TableLayout tl = new TableLayout();
	        		tl.addColumnData(new ColumnWeightData(1));
	        		tl.addColumnData(new ColumnWeightData(1));
	        		table.setLayout(tl);
	        		table.layout();
	        		// only do this once
	        		table.removePaintListener(this);
	        	}
	        });
	}
	
	private void createTableViewer() {
        tableViewer = new TableViewer(table);
        tableViewer.setColumnProperties(new String[] {
                IArgsConstants.TABLE_COL_TYPE,
                IArgsConstants.TABLE_COL_ARGUMENT });

        CellEditor[] editors = new CellEditor[2];
        editors[0] = new TextCellEditor(table);
        editors[1] = new TextCellEditor(table);
        // keep track of when the cell loses focus
        ((Text) editors[1].getControl()).addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                fireParameterChangedEvent();
            }
        });
        tableViewer.setCellEditors(editors);
        tableViewer.setCellModifier(new ArgsCellModifier(tableViewer));
        tableViewer.setContentProvider(new ArgsContentProvider());
        tableViewer.setLabelProvider(new ArgsLabelProvider());
	}
	
	private void fireParameterChangedEvent() {
        Object[] listenerArray = listeners.getListeners();
        for (int i = 0; i < listenerArray.length; ++i)
            ((ParameterChangeListener) listenerArray[i]).parameterChanged();
    }

	/**
	 * @return Returns the tableViewer
	 */
	public TableViewer getTableViewer() {
		return tableViewer;
	}
    
    /**
     * turns an array of classes into an array of argument pairs
     * 
     * @param classes
     *            the parameter types of a constructor
     * @return the given argument pairs for these classes
     * @see Parameter
     */
    public static Parameter[] toParameters(IClass[] classes) {
        Parameter[] wrappers = new Parameter[classes.length];
        for (int i = 0; i < classes.length; i++)
            wrappers[i] = new Parameter(classes[i]);
        return wrappers;
    }
    
    public void addParameterChangeListener(ParameterChangeListener listener) {
        listeners.add(listener);
    }
    
    public void removeParameterChangeListener(ParameterChangeListener listener) {
        listeners.remove(listener);
    }
}
