/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 19, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: AbstractNode.java,v 1.1 2006/01/23 16:05:49 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

import edu.wpi.ebob.model.cm.IObject;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public abstract class AbstractNode implements Node {

    /** the children of this node */
    protected List children;

    /** the parent of this node */
    private Node parent;

    /** constructs a new general-type node. */
    public AbstractNode() {
        children = new ArrayList();
    }

    /**
     * adds <code>child</code> as a child node of the current node, and sets
     * its parent to <code>this</code>.
     */
    protected void addChild(Node child) {
        children.add(child);
        child.setParent(this);
    }

    /**
     * @see Node#getChildren()
     */
    public Node[] getChildren() {
    	Node[] childrenArray = new Node[children.size()];
        return (Node[]) children.toArray(childrenArray);
    }

    /**
     * @see Node#getParent()
     */
    public Node getParent() {
        return parent;
    }

    /**
     * returns the toString() of {@link Node#getValue()}.
     * 
     * @see Node#getValueAsString()
     */
    public String getValueAsString() {
        IObject value = getValue();
        return (null != value) ? value.toString() : ""; //$NON-NLS-1$
    }

    /**
     * @see Node#hasChildren()
     */
    public boolean hasChildren() {
        return (children.size() > 0);
    }

    /**
     * 
     * @see Node#setParent(Node)
     */
    public void setParent(Node parent) {
        this.parent = parent;
    }

    /**
     * @return <code>null</code>
     * @see edu.wpi.ebob.internal.inspect.Node#getImage()
     */
    public Image getImage() {
        return null;
    }

    /**
     * @return {@link Node#getName()}.
     */
    public String toString() {
        return this.getName();
    }
}