/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 20, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: InstanceContentProvider.java,v 1.1 2006/01/23 16:05:49 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import java.text.MessageFormat;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import edu.wpi.ebob.ObjBenchPlugin;

/**
 * a content provider for the TableTreeViewer in
 * {@link edu.wpi.ebob.views.PropertiesView}.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class InstanceContentProvider implements IStructuredContentProvider,
        ITreeContentProvider {

    /**
     * does nothing.
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(Viewer,
     *      Object, Object)
     */
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
    }

    /**
     * does nothing.
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose() {
    }

    /**
     * @see IStructuredContentProvider#getElements(Object)
     */
    public Object[] getElements(Object parent) {
        return getChildren(parent);
    }

    /**
     * @see ITreeContentProvider#getParent(Object)
     */
    public Object getParent(Object child) {
        if (child instanceof Node)
            return ((Node) child).getParent();
        throw new AssertionError(MessageFormat.format(ObjBenchPlugin
                .getResourceString("InstanceContentProvider.badChild"), //$NON-NLS-1$
                new Object[] { Node.class }));
    }

    /**
     * @see ITreeContentProvider#getChildren(Object)
     */
    public Object[] getChildren(Object parent) {
        if (parent instanceof Node)
            return ((Node) parent).getChildren();
        throw new AssertionError(MessageFormat.format(ObjBenchPlugin
                .getResourceString("InstanceContentProvider.badParent"), //$NON-NLS-1$
                new Object[] { Node.class }));
    }

    /**
     * @see ITreeContentProvider#hasChildren(Object)
     */
    public boolean hasChildren(Object parent) {
        if (parent instanceof Node)
            return ((Node) parent).hasChildren();
        throw new AssertionError(MessageFormat.format(ObjBenchPlugin
                .getResourceString("InstanceContentProvider.badParent"), //$NON-NLS-1$
                new Object[] { Node.class }));
    }
}