/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Initially created on Dec 20, 2004
 * 
 * Contributors: Liam Morley - initial API and implementation
 * 
 * $Id: InstanceLabelProvider.java,v 1.1 2006/01/23 16:05:49 bigben Exp $
 ******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.cm.IClass;

/**
 * A label provider for the TableTreeViewer in
 * {@link edu.wpi.ebob.views.PropertiesView}.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class InstanceLabelProvider extends LabelProvider {
    /**
     * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
     */
    public Image getImage(Object element) {
        if (element instanceof Node)
            return ((Node) element).getImage();
        return null;
    }

    /**
     * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
     */
    public String getText(Object element) {
        if (element instanceof Node) {
            Node node = (Node) element;
            StringBuffer sb = new StringBuffer();
            if (!node.getClass().equals(ArrayItemNode.class)) {
                IClass type = node.getType();
                if (null != type)
                    if (type.isArray())
                        sb.append(type.getComponentType().toString() + "[] "); //$NON-NLS-1$
                    else
                        sb.append(type.toString() + " "); //$NON-NLS-1$
            }
            sb.append(node.getName());
            String value = node.getValueAsString();
            if (!"".equals(value)) //$NON-NLS-1$
                sb.append(" = " + value); //$NON-NLS-1$
            return sb.toString();
        }
        throw new AssertionError(
                ObjBenchPlugin
                        .getResourceString("InstanceLabelProvider.treeElementsMustBeNodes")); //$NON-NLS-1$
    }
}