/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Initially created on Dec 19, 2004
 * 
 * Contributors: Liam Morley - initial API and implementation
 * 
 * $Id: InstanceNode.java,v 1.1 2006/01/23 16:05:49 bigben Exp $
 ******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.internal.gui.Instance;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IField;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * represents an {@link Instance}in a tree.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class InstanceNode extends AbstractNode {

    /** implements lazy checking. */
    private boolean hasCheckedForChildren = false;

    /** the underlying instance. */
    private Instance instance;

    /**
     * @param instance
     *            the underlying instance for this node.
     */
    public InstanceNode(Instance instance) {
        this.instance = instance;
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getChildren()
     */
    public Node[] getChildren() {
        if (!hasCheckedForChildren) {
            hasCheckedForChildren = true;
            IObject obj = instance.getObject();
            if (null != obj) {
                IField[] fields = null;
                try {
                    fields = obj.getIClass().getDeclaredFields();
                } catch (RemoteException re) {
                    DetailsDialogUtil.openError(Display.getCurrent()
                            .getActiveShell(), re);
                    ObjBenchLogger.logError(re.getMessage(), re);
                }
                for (int i = 0; i < fields.length; i++)
                    addChild(new FieldNode(obj, fields[i]));
            }
        }
        return super.getChildren();
    }

    /**
     * @see edu.wpi.ebob.internal.inspect.Node#getImage()
     */
    public Image getImage() {
        return JavaUI.getSharedImages().getImage(
                ISharedImages.IMG_OBJS_LOCAL_VARIABLE);
    }

    /**
     * @see Node#getName()
     */
    public String getName() {
        return instance.getInstanceName();
    }

    /**
     * @see Node#getType()
     */
    public IClass getType() {
        return instance.getObject().getIClass();
    }

    /**
     * returns <code>null</code> as a value for an instance node doesn't make
     * sense; the "value" is the children.
     * 
     * @see Node#getValue()
     */
    public IObject getValue() {
        return null;
    }

    /**
     * @see Node#hasChildren()
     */
    public boolean hasChildren() {
        Object obj = instance.getObject();
        return (null != obj && obj.getClass().getDeclaredFields().length > 0);
    }
}