/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 19, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: Node.java,v 1.1 2006/01/23 16:05:49 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.inspect;

import org.eclipse.swt.graphics.Image;

import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IObject;

/**
 * Represents a node in a tree.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public interface Node {
    /** the text label for this node. */
    public String getName();

    /** the underlying value for this node. */
    public IObject getValue();

    /** a textual representation of the value. */
    public String getValueAsString();

    /** the parent node of this node. */
    public Node getParent();

    /** sets the parent of this node to be <code>parent</code>. */
    public void setParent(Node parent);

    /**
     * @return <code>true</code> if this node has children, <code>false</code>
     *         otherwise.
     */
    public boolean hasChildren();

    /** returns the child nodes of this node, represented as objects. */
    public Node[] getChildren();

    /** returns the image that should be shown for this node. */
    public Image getImage();

    /** returns the <code>Class</code> object associated with this node. */
    public IClass getType();
}