/*******************************************************************************
 * Copyright (c) 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Feb 21, 2005
 *
 * Contributors:
 *    Justin Braga - initial API and implementation
 *    Liam Morley - added set() method
 * 
 * $Id: JEMArray.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import org.eclipse.jem.internal.proxy.core.IArrayBeanProxy;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;

import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IArray;
import edu.wpi.ebob.model.cm.IObject;

/**
 * @author <a href="mailto:trevelyan2001@users.sourceforge.net">Justin Braga
 *         </a>
 *  
 */
public class JEMArray extends JEMObject implements IArray {

    private IArrayBeanProxy jemArray;

    /**
     * @param jemArray
     */
    JEMArray(IArrayBeanProxy jemArray) {
        super(jemArray);
        this.jemArray = jemArray;
    }

    /**
     * @see IArray#get(int)
     */
    public IObject get(int index) throws RemoteException {
        try {
            return JEMObject.getObject(jemArray.get(index));
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see IArray#set(IObject, int)
     */
    public void set(IObject value, int index) throws RemoteException {
        try {
            jemArray.set(((JEMObject) value).jemObject, index);
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see IArray#getLength()
     */
    public int getLength() {
        return jemArray.getLength();
    }
}