/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 29, 2005
 *
 * Contributors:
 *		Liam Morley - initial API and implementation 
 *		Justin Braga - Implemented getParameterTypes
 *		Liam Morley - made this extend JEMObject
 *		Justin Braga - newInstance and getParameterTypes now throw CMException
 *		Justin Braga - Using simple class names in toString
 *		Justin Braga - Moved out some toString logic into JEMClass   
 *	
 * $Id: JEMConstructor.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.jem.internal.proxy.core.IArrayBeanProxy;
import org.eclipse.jem.internal.proxy.core.IBeanProxy;
import org.eclipse.jem.internal.proxy.core.IBeanTypeProxy;
import org.eclipse.jem.internal.proxy.core.IConstructorProxy;
import org.eclipse.jem.internal.proxy.core.IMethodProxy;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IConstructor;
import edu.wpi.ebob.model.cm.IObject;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class JEMConstructor extends JEMMember implements IConstructor {

    private static Map constructors = new WeakHashMap();

    private static IMethodProxy getParameterTypes;

    static JEMConstructor getConstructor(IConstructorProxy jemConstructor,
            IBeanTypeProxy parent) {
        if (constructors.containsKey(jemConstructor))
            return (JEMConstructor) constructors.get(jemConstructor);
        JEMConstructor constructor = new JEMConstructor(jemConstructor, parent);
        constructors.put(jemConstructor, constructor);
        return constructor;
    }

    private String cachedToString;

    private IConstructorProxy jemConstructor;

    private IBeanTypeProxy parent;

    /**
     * @param jemConstructor
     */
    private JEMConstructor(IConstructorProxy jemConstructor,
            IBeanTypeProxy parent) {
        super(jemConstructor);
        this.jemConstructor = jemConstructor;
        this.parent = parent;
    }

    /**
     * @see IObject#getIClass()
     */
    public IClass getIClass() {
        return JEMClass.getClass(parent);
    }

    /**
     * @see edu.wpi.ebob.model.cm.IConstructor#getParameterTypes()
     */
    public IClass[] getParameterTypes() throws RemoteException {
        if (null == getParameterTypes || !getParameterTypes.isValid())
            getParameterTypes = jemConstructor.getTypeProxy().getMethodProxy(
                    "getParameterTypes"); //$NON-NLS-1$
        IArrayBeanProxy beanParmTypes;
        try {
            beanParmTypes = (IArrayBeanProxy) (getParameterTypes
                    .invoke(jemConstructor));
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }

        IClass[] parmTypes = new IClass[beanParmTypes.getLength()];
        for (int i = 0; i < beanParmTypes.getLength(); i++) {
            try {
                parmTypes[i] = JEMClass.getClass((IBeanTypeProxy) beanParmTypes
                        .get(i));
            } catch (ThrowableProxy tp) {
                throw new JEMRemoteException(tp);
            }
        }
        return parmTypes;
    }

    /**
     * @throws RemoteException
     * @see IConstructor#newInstance(IObject[])
     */
    public IObject newInstance(IObject[] initargs) throws RemoteException {
        IBeanProxy[] args = new IBeanProxy[initargs.length];
        for (int i = 0; i < initargs.length; i++) {
            JEMObject arg = (JEMObject) initargs[i];
            args[i] = (null != arg) ? arg.getBeanProxy() : null;
        }
        try {
            return JEMObject.getObject(jemConstructor.newInstance(args));
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        if (cachedToString != null)
            return cachedToString;

        StringBuffer sb = new StringBuffer();
        sb.append(getIClass());
        sb.append("("); //$NON-NLS-1$

        try {
            IClass[] types = this.getParameterTypes();
            for (int i = 0; i < types.length; i++) {
                sb.append(types[i].toString());
                if (i < types.length - 1)
                    sb.append(", "); //$NON-NLS-1$
            }
        } catch (RemoteException cme) {
            ObjBenchLogger.logError(cme.getMessage(), cme);
        }

        sb.append(")"); //$NON-NLS-1$
        return sb.toString();
    }
}