/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 29, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: JEMField.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.jem.internal.proxy.core.IBeanProxy;
import org.eclipse.jem.internal.proxy.core.IFieldProxy;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IField;
import edu.wpi.ebob.model.cm.IObject;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class JEMField extends JEMMember implements IField {

    private IFieldProxy jemField;

    private static Map fields = new WeakHashMap();

    private JEMField(IFieldProxy jemField) {
        super(jemField);
        this.jemField = jemField;
        try {
            this.jemField.setAccessible(true);
        } catch (ThrowableProxy tp) {
            // not much we can do in this case
            ObjBenchLogger.logWarning(tp);
        }
    }

    /**
     * @throws RemoteException
     * @see IField#get(IObject)
     */
    public IObject get(IObject obj) throws RemoteException {
        try {
            return JEMObject.getObject(jemField.get(((JEMObject) obj)
                    .getBeanProxy()));
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @throws RemoteException
     * @see IField#set(IObject, IObject)
     */
    public void set(IObject obj, IObject value) throws RemoteException {
        try {
            // value can be null to start with
            IBeanProxy newValue = (null != value) ? ((JEMObject) value)
                    .getBeanProxy() : null;
            jemField.set(((JEMObject) obj).getBeanProxy(), newValue);
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see edu.wpi.ebob.model.cm.IField#getType()
     */
    public IClass getType() {
        return JEMClass.getClass(jemField.getFieldType());
    }

    static JEMField getField(IFieldProxy jemField) {
        if (null == jemField)
            return null;
        if (fields.containsKey(jemField))
            return (JEMField) fields.get(jemField);
        JEMField field = new JEMField(jemField);
        fields.put(jemField, field);
        return field;
    }
}