/*******************************************************************************
 * Copyright (c) 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Feb 18, 2005
 *
 * Contributors:
 *    Justin Braga - initial API and implementation
 * 
 * $Id: JEMMember.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.lang.reflect.Modifier;

import org.eclipse.jem.internal.proxy.core.IBeanProxy;
import org.eclipse.jem.internal.proxy.core.IMethodProxy;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;

import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IMember;

/**
 * Class implementing methods common to constructors, fields, and methods
 * 
 * @author <a href="mailto:trevelyan2001@users.sourceforge.net">Justin Braga
 *         </a>
 *  
 */
public class JEMMember extends JEMObject implements IMember {
    private IMethodProxy getModifiers;

    private IMethodProxy getName;

    /**
     * @param jemMember
     */
    protected JEMMember(IBeanProxy jemMember) {
        super(jemMember);
    }

    /**
     * @see IMember#getName()
     */
    public String getName() throws RemoteException {
        if (null == getName || !getName.isValid())
            getName = jemObject.getTypeProxy().getMethodProxy("getName"); //$NON-NLS-1$
        try {
            IBeanProxy proxyResult = getName.invoke(jemObject);
            return proxyResult.toBeanString();
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * Retrieves the modifiers which are used by the other methods.
     * 
     * @throws RemoteException
     */
    private int getModifiers() throws RemoteException {
        if (null == getModifiers || !getModifiers.isValid())
            getModifiers = jemObject.getTypeProxy().getMethodProxy(
                    "getModifiers"); //$NON-NLS-1$
        try {
            IBeanProxy proxResult = getModifiers.invoke(jemObject);
            return Integer.parseInt(proxResult.toBeanString());
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMember#isAbstract()
     */
    public boolean isAbstract() throws RemoteException {
        return Modifier.isAbstract(getModifiers());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMember#isPublic()
     */
    public boolean isPublic() throws RemoteException {
        return Modifier.isPublic(getModifiers());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMember#isProtected()
     */
    public boolean isProtected() throws RemoteException {
        return Modifier.isProtected(getModifiers());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMember#isPrivate()
     */
    public boolean isPrivate() throws RemoteException {
        return Modifier.isPrivate(getModifiers());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMember#isStatic()
     */
    public boolean isStatic() throws RemoteException {
        return Modifier.isStatic(getModifiers());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMember#isFinal()
     */
    public boolean isFinal() throws RemoteException {
        return Modifier.isFinal(getModifiers());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMember#isSynchronized()
     */
    public boolean isSynchronized() throws RemoteException {
        return Modifier.isSynchronized(getModifiers());
    }
}