/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 29, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: JEMMethod.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.jem.internal.proxy.core.IBeanProxy;
import org.eclipse.jem.internal.proxy.core.IBeanTypeProxy;
import org.eclipse.jem.internal.proxy.core.IMethodProxy;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;

import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IMethod;
import edu.wpi.ebob.model.cm.IObject;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class JEMMethod extends JEMMember implements IMethod {
    private IMethodProxy jemMethod;

    private static Map methods = new WeakHashMap();

    private JEMMethod(IMethodProxy jemMethod) {
        super(jemMethod);
        this.jemMethod = jemMethod;
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMethod#getDeclaringClass()
     */
    public IClass getDeclaringClass() {
        return JEMClass.getClass(jemMethod.getClassType());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMethod#getName()
     */
    public String getName() {
        return jemMethod.getName();
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMethod#getReturnType()
     */
    public IClass getReturnType() {
        return JEMClass.getClass(jemMethod.getReturnType());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IMethod#getParameterTypes()
     */
    public IClass[] getParameterTypes() {
        IBeanTypeProxy[] parameterTypes = jemMethod.getParameterTypes();
        IClass[] classes = new IClass[parameterTypes.length];
        for (int i = 0; i < parameterTypes.length; i++)
            classes[i] = JEMClass.getClass(parameterTypes[i]);
        return classes;
    }

    /**
     * @see IMethod#invoke(IObject, IObject[])
     */
    public IObject invoke(IObject obj, IObject[] args) throws RemoteException {
        IBeanProxy[] arguments = new IBeanProxy[args.length];
        for (int i = 0; i < args.length; i++) {
            JEMObject arg = (JEMObject) args[i];
            arguments[i] = (null != arg) ? arg.getBeanProxy() : null;
        }
        try {
            return JEMObject.getObject(jemMethod.invoke(((JEMObject) obj)
                    .getBeanProxy(), arguments));
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        }
    }

    static JEMMethod getMethod(IMethodProxy jemMethod) {
        if (null == jemMethod)
            return null;
        if (methods.containsKey(jemMethod))
            return (JEMMethod) methods.get(jemMethod);
        JEMMethod method = new JEMMethod(jemMethod);
        methods.put(jemMethod, method);
        return method;
    }
}