/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Jan 29, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: JEMObject.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.jem.internal.proxy.core.IArrayBeanProxy;
import org.eclipse.jem.internal.proxy.core.IBeanProxy;

import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IObject;

/**
 * Represents a remote object in a JEM JVM.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class JEMObject implements IObject {
	protected IBeanProxy jemObject;

	private static Map objects = new WeakHashMap();

	protected JEMObject(IBeanProxy jemObj) {
		this.jemObject = jemObj;
	}

	/**
	 * @see edu.wpi.ebob.model.cm.IObject#getIClass()
	 */
	public IClass getIClass() {
		return JEMClass.getClass(jemObject.getTypeProxy());
	}

	/**
	 * @see IObject#isInstanceof(IClass)
	 */
	public boolean isInstanceof(IClass obj) {
		return jemObject.getTypeProxy().isKindOf(
				((JEMClass) obj).getBeanTypeProxy());
	}

	/**
	 * @see edu.wpi.ebob.model.cm.IObject#objToString()
	 */
	public String objToString() {
		return jemObject.toBeanString();
	}

	public IBeanProxy getBeanProxy() {
		return jemObject;
	}

	/**
	 * @see Object#toString()
	 */
	public String toString() {
		//System.out.println(jemObject.toString());
		return jemObject.toBeanString();
	}

	public static final IObject getObject(IBeanProxy jemObject) {
		if (null == jemObject)
			return null;
		if (objects.containsKey(jemObject))
			return (IObject) objects.get(jemObject);
		JEMObject obj;
		if (jemObject instanceof IArrayBeanProxy)
			obj = new JEMArray((IArrayBeanProxy) jemObject);
		else
			obj = new JEMObject(jemObject);
		objects.put(jemObject, obj);
		return obj;
	}
}