/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Mar 26, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: JEMRemoteException.java,v 1.1 2006/01/23 16:06:01 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem;

import java.io.PrintStream;
import java.io.PrintWriter;

import org.eclipse.jem.internal.proxy.core.ThrowableProxy;

import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * Represents an exception thrown on a JEM remote JVM.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class JEMRemoteException extends RemoteException {
    private ThrowableProxy tp;

    /**
     * Constructs a new JEMRemoteException using a JEM ThrowableProxy.
     */
    public JEMRemoteException(ThrowableProxy tp) {
        this.tp = tp;
    }

    /**
     * @see edu.wpi.ebob.model.cm.IObject#getIClass()
     */
    public IClass getIClass() {
        return JEMClass.getClass(tp.getTypeProxy());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IObject#isInstanceof(IClass)
     */
    public boolean isInstanceof(IClass obj) {
        return tp.getTypeProxy().isKindOf(((JEMClass) obj).getBeanTypeProxy());
    }

    /**
     * @see edu.wpi.ebob.model.cm.IObject#objToString()
     */
    public String objToString() {
        return tp.toBeanString();
    }

    /**
     * @see Throwable#getLocalizedMessage()
     */
    public String getLocalizedMessage() {
        return tp.getProxyLocalizedMessage();
    }

    /**
     * @see java.lang.Throwable#getMessage()
     */
    public String getMessage() {
        return tp.getProxyMessage();
    }

    /**
     * @see Throwable#printStackTrace()
     */
    public void printStackTrace() {
        tp.printProxyStackTrace();
    }

    /**
     * @see Throwable#printStackTrace(PrintStream)
     */
    public void printStackTrace(PrintStream s) {
        tp.printProxyStackTrace(s);
    }

    /**
     * @see Throwable#printStackTrace(PrintWriter)
     */
    public void printStackTrace(PrintWriter s) {
        tp.printProxyStackTrace(s);
    }

    /**
     * @see Object#toString()
     */
    public String toString() {
        String toString = tp.getTypeProxy().getFormalTypeName();
        String message = tp.getLocalizedMessage();
        if (null != message)
            toString += ": " + message; //$NON-NLS-1$
        return toString;
    }
}
