/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Feb 18, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: JEMAbstractInfixOperation.java,v 1.1 2006/01/23 16:06:39 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem.operations;

import org.eclipse.jem.internal.proxy.core.IExpression;
import org.eclipse.jem.internal.proxy.core.ProxyFactoryRegistry;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;
import org.eclipse.jem.internal.proxy.initParser.tree.ForExpression;
import org.eclipse.jem.internal.proxy.initParser.tree.InfixOperator;
import org.eclipse.jem.internal.proxy.initParser.tree.NoExpressionValueException;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.jem.JEMClassManager;
import edu.wpi.ebob.internal.jem.JEMObject;
import edu.wpi.ebob.internal.jem.JEMRemoteException;
import edu.wpi.ebob.model.cm.IClassManager;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * Performs an infix argument accepting two arguments.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public abstract class JEMAbstractInfixOperation {
    protected static final int PLUS = InfixOperator.IN_PLUS_VALUE;

    protected static final int MINUS = InfixOperator.IN_MINUS_VALUE;

    protected static final int STAR = InfixOperator.IN_TIMES_VALUE;

    protected static final int SLASH = InfixOperator.IN_DIVIDE_VALUE;

    protected static final int MOD = InfixOperator.IN_REMAINDER_VALUE;

    /**
     * Performs operation <code>operator</code> on arguments <code>args</code>.
     * There must only be 2 arguments.
     * 
     * @param cm
     *            A valid {@link JEMClassManager}.
     * @param args
     *            the arguments
     * @param operator
     *            the operator to perform
     * @return the result of the operation
     * @throws RemoteException
     */
    protected IObject performOperation(IClassManager cm, IObject[] args,
            int operator) throws RemoteException {
    	if (args.length != 2)
            throw new IllegalArgumentException(ObjBenchPlugin.getResourceString("JEMAbstractInfixOperation.badArgsLength")); //$NON-NLS-1$
        ProxyFactoryRegistry registry = ((JEMClassManager) cm).getRegistry();
        IExpression expr = registry.getBeanProxyFactory().createExpression();
        try {
        	expr.createInfixExpression(ForExpression.ROOTEXPRESSION,
        			InfixOperator.get(operator), 0);
            expr.createProxyExpression(ForExpression.INFIX_LEFT,
                    ((JEMObject) args[0]).getBeanProxy());
            expr.createProxyExpression(ForExpression.INFIX_RIGHT,
                    ((JEMObject) args[1]).getBeanProxy());
            return JEMObject.getObject(expr.getExpressionValue());
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
		} catch (NoExpressionValueException neve) {
			return null;
		}
    }
}