/*******************************************************************************
 * Copyright (c) 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Apr 1, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: JEMAbstractPrefixOperation.java,v 1.1 2006/01/23 16:06:39 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.jem.operations;

import org.eclipse.jem.internal.proxy.core.IExpression;
import org.eclipse.jem.internal.proxy.core.ProxyFactoryRegistry;
import org.eclipse.jem.internal.proxy.core.ThrowableProxy;
import org.eclipse.jem.internal.proxy.initParser.tree.ForExpression;
import org.eclipse.jem.internal.proxy.initParser.tree.NoExpressionValueException;
import org.eclipse.jem.internal.proxy.initParser.tree.PrefixOperator;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.internal.jem.JEMClassManager;
import edu.wpi.ebob.internal.jem.JEMObject;
import edu.wpi.ebob.internal.jem.JEMRemoteException;
import edu.wpi.ebob.model.cm.IClassManager;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.cm.IOperation;
import edu.wpi.ebob.model.cm.RemoteException;

/**
 * Performs an infix argument accepting two arguments.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public abstract class JEMAbstractPrefixOperation implements IOperation {
    protected static final int PLUS = PrefixOperator.PRE_PLUS_VALUE;

    protected static final int MINUS = PrefixOperator.PRE_MINUS_VALUE;

    protected static final int NOT = PrefixOperator.PRE_NOT_VALUE;

    protected static final int COMPLIMENT = PrefixOperator.PRE_COMPLEMENT_VALUE;

    /**
     * @see edu.wpi.ebob.model.cm.IOperation#performOperation(edu.wpi.ebob.model.cm.IClassManager,
     *      edu.wpi.ebob.model.cm.IObject[])
     */
    protected IObject performOperation(IClassManager cm, IObject[] args,
            int operator) throws RemoteException {
        if (args.length > 1)
            throw new IllegalArgumentException("can't have more than 1 arg");
        ProxyFactoryRegistry registry = ((JEMClassManager) cm).getRegistry();
        IExpression expr = registry.getBeanProxyFactory().createExpression();
        try {
            expr.createPrefixExpression(ForExpression.ROOTEXPRESSION,
                    PrefixOperator.get(operator));
            expr.createProxyExpression(ForExpression.PREFIX_OPERAND,
                    ((JEMObject) args[0]).getBeanProxy());
            return JEMObject.getObject(expr.getExpressionValue());
        } catch (IllegalStateException ise) {
            ObjBenchLogger.logError(ise);
            return null;
        } catch (ThrowableProxy tp) {
            throw new JEMRemoteException(tp);
        } catch (NoExpressionValueException neve) {
            ObjBenchLogger.logWarning(
                    "function did not return value, but was supposed to", neve);
            return null;
        }
    }
}