/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 8, 2004
 *
 * Contributors:
 * 	  Justin Braga - Replaced all references of Argument with new Parameter
 *    Liam Morley - initial API and implementation
 * 
 * $Id: ArgsCellModifier.java,v 1.1 2006/01/23 16:07:58 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.wizards;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Item;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.Parameter;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 * @see ICellModifier
 */
public class ArgsCellModifier implements ICellModifier {
    private final TableViewer viewer;

    /**
     * @param viewer
     *            the viewer to update after making modifications
     */
    public ArgsCellModifier(TableViewer viewer) {
        this.viewer = viewer;
    }

    /**
     * @see ICellModifier#canModify(Object, String)
     */
    public boolean canModify(Object element, String property) {
        return (property.equals(IArgsConstants.TABLE_COL_ARGUMENT));
    }

    /**
     * @see ICellModifier#getValue(Object, String)
     */
    public Object getValue(Object element, String property) {
        try {
            Parameter arg = (Parameter) element;
            if (property.equals(IArgsConstants.TABLE_COL_TYPE))
                return arg.getReturnType();
            else if (property.equals(IArgsConstants.TABLE_COL_ARGUMENT)) {
                Object obj = arg.getInput();
                return (null != obj) ? obj.toString() : ""; //$NON-NLS-1$
            } else
                return null;
        } catch (Throwable t) {
            t.printStackTrace();
            return null;
        }
    }

    /**
     * @see ICellModifier#modify(Object, String, Object)
     */
    public void modify(Object element, String property, Object value) {
        if (!property.equals(IArgsConstants.TABLE_COL_ARGUMENT)) {
            ObjBenchLogger
                    .logWarning(ObjBenchPlugin.getResourceString("ArgsCellModifier.nonArgEdit")); //$NON-NLS-1$
            return;
        }
        Parameter arg;
        if (element instanceof Item)
            arg = (Parameter) ((Item) element).getData();
        else
            arg = (Parameter) element;
        arg.setInput(value.toString());
        viewer.refresh();
    }
}