/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 8, 2004
 *
 * Contributors:
 * 	  Justin Braga - Replaced all references of Argument with new Parameter
 *    Liam Morley - initial API and implementation
 * 
 * $Id: ArgsContentProvider.java,v 1.1 2006/01/23 16:07:58 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.internal.wizards;

import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;

import edu.wpi.ebob.model.Parameter;

/**
 * InnerClass that acts as a proxy for the ExampleTaskList providing content for
 * the Table. It implements the ITaskListViewer interface since it must register
 * changeListeners with the ExampleTaskList
 */
public class ArgsContentProvider implements IStructuredContentProvider {
    Parameter[] args;

    /**
     * @see IContentProvider#inputChanged(Viewer, Object, Object)
     */
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
        args = (Parameter[]) newInput;
    }

    /**
     * @see IContentProvider#dispose()
     */
    public void dispose() {
        args = null;
    }

    /**
     * Returns the arguments within the table. The form of these arguments
     * depends on the value of <code>parent</code>. If <code>parent</code>
     * is <code>null</code>, then only the argument values are returned; if
     * an input is specified, then that input is formatted as an array of
     * Arguments and returned as such.
     * 
     * @see IStructuredContentProvider#getElements(Object)
     */
    public Object[] getElements(Object parent) {
        return (Parameter[]) parent;
    }
}