package edu.wpi.ebob.model;

/**
 * Represents a parameter to a method call, and stores the fully qualified object name
 * as well as the instance name of the parameter.
 * 
 * @author bigben
 * @date   11/30/05
 */
public class MethodInvokeParamInfo {
	private String fullyQualifiedObjectName;
	private String instanceName;
	
	/**
	 * Constructs a MethodInvokeParam object.
	 * @param fqon the fully qualified object name of the parameter, such as "java.lang.String"
	 * @param in   the instance name of the actual parameter.
	 */
	public MethodInvokeParamInfo(String fqon, String in) {
		fullyQualifiedObjectName = fqon;
		instanceName = in;
	}

	/**
	 * Gets the fully qualified object name.
	 * @return the fully qualified object name.
	 */
	public String getFullyQualifiedObjectName() {
		return fullyQualifiedObjectName;
	}

	/**
	 * Sets the fully qualified object name.
	 * @param fullyQualifiedObjectName the fully qualified object name to set.
	 */
	public void setFullyQualifiedObjectName(String fullyQualifiedObjectName) {
		this.fullyQualifiedObjectName = fullyQualifiedObjectName;
	}

	/**
	 * Gets the name of the instance of the object.
	 * @return the name of the instance of the object.
	 */
	public String getParamObjectInstanceName() {
		return instanceName;
	}

	/**
	 * Sets the name of the instance of the object.
	 * @param instanceName the instanceName to set.
	 */
	public void setParamObjectInstanceName(String instanceName) {
		this.instanceName = instanceName;
	}

	
}
