/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 24, 2004
 *
 * Contributors:
 *    Administrator - initial API and implementation
 * 
 * $Id: ObjectRemovalNotSupportedException.java,v 1.1 2006/01/23 16:07:41 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.model;

import java.util.ArrayList;
import java.util.Collection;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class ObjectRemovalNotSupportedException extends Exception {
    private Collection names;

    public ObjectRemovalNotSupportedException(Collection names) {
        this.names = names;
    }

    public ObjectRemovalNotSupportedException(String name) {
        init(name);
    }

    /**
     * initializes the exception with the given name.
     * 
     * @param name
     *            the name of the object that can't be removed.
     */
    private void init(String name) {
        names = new ArrayList(2);
        names.add(name);
    }

    public ObjectRemovalNotSupportedException(String name, Throwable cause) {
        super(cause);
        init(name);
    }

    /**
     * @param names
     * @param cause
     */
    public ObjectRemovalNotSupportedException(Collection names, Throwable cause) {
        super(cause);
        this.names = names;
    }

    /**
     * Returns the names of the instances in the Object Bench that were unable
     * to be removed. This is designed to be expended just once, and as such
     * does not return an unmodifiable Collection.
     * 
     * @return the Collection of names unable to be removed.
     */
    public Collection getNames() {
        return names;
    }
}