/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Feb 7, 2005
 *
 * Contributors:
 * 	  Justin Braga = Added getter for returnType
 *    Liam Morley - initial API and implementation
 * 
 * $Id: Parameter.java,v 1.1 2006/01/23 16:07:41 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.model;

import java.util.Collection;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.parser.Parser;
import edu.wpi.ebob.model.parser.ParserException;
import edu.wpi.ebob.model.parser.Scanner;

/**
 * Represents an input parameter for a method or constructor, or the input value
 * for a field.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class Parameter {
    private IClass returnType;

    private String input;

    private Scanner scanner = new Scanner();

    private Collection tokens;

    private IObject result = null;

    /** Constructs a new Parameter with the given return type. */
    public Parameter(IClass returnType) {
        this.returnType = returnType;
    }

    /** sets the value of this parameter to the user-input string. */
    public void setInput(String input) {
        this.input = input;
        this.result = null;
        tokens = scanner.scan(input);
    }

    /**
     * @return Returns the returnType.
     */
    public IClass getReturnType() {
        return returnType;
    }

    public String getInput() {
        return input;
    }

    /**
     * evaluates the user input to an IObject.
     * 
     * @return an IObject if the input string is valid.
     * @throws IllegalStateException
     *             if the input has not yet been set.
     * @throws ParserException
     *             if the input string is invalid
     * @throws ClassCastException
     *             if the input string does not resolve to an instance of
     *             returnType or any of its superclasses
     */
    public IObject evaluate() throws ParserException {
        if (input == null)
            throw new IllegalStateException(ObjBenchPlugin
                    .getResourceString("Parameter.nullInput")); //$NON-NLS-1$
        if (result == null) {
            Parser parser = new Parser();
            result = parser.parse(tokens);
            if ((null != result && !result.isInstanceof(returnType))
                    || (null == result && returnType.isPrimitive()))
                throw new ClassCastException();
        }
        return result;
    }
}