/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Feb 11, 2005
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: TokenType.java,v 1.1 2006/01/23 16:08:23 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.model.parser;

/**
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 *  
 */
public class TokenType {
    private String tokenValue;

    /** token signifying a java identifier */
    static final public TokenType ID = new TokenType("Identifier"); //$NON-NLS-1$

    // primitive types
    /** token signifying a primitive char */
    static final public TokenType CHAR = new TokenType("char"); //$NON-NLS-1$

    /** token signifying a primitive int */
    static final public TokenType INT = new TokenType("int"); //$NON-NLS-1$

    /** token signifying a primitive float */
    public static final TokenType FLOAT = new TokenType("float"); //$NON-NLS-1$

    /** token signifying a java.lang.String object */
    static final public TokenType STRING = new TokenType("java.lang.String"); //$NON-NLS-1$

    // keyword
    /** token signifying a keyword */
    static final public TokenType KEYWORD = new TokenType("keyword"); //$NON-NLS-1$

    // separators
    /** '(' token */
    static final public TokenType LPAR = new TokenType("("); //$NON-NLS-1$

    /** ')' token */
    static final public TokenType RPAR = new TokenType(")"); //$NON-NLS-1$

    /** '{' token */
    static final public TokenType LCURLY = new TokenType("{"); //$NON-NLS-1$

    /** '}' token */
    static final public TokenType RCURLY = new TokenType("}"); //$NON-NLS-1$

    /** '[' token */
    static final public TokenType LBRACK = new TokenType("["); //$NON-NLS-1$

    /** ']' token */
    static final public TokenType RBRACK = new TokenType("]"); //$NON-NLS-1$

    /** '.' token */
    static final public TokenType DOT = new TokenType("."); //$NON-NLS-1$

    /** ',' token */
    static final public TokenType COMMA = new TokenType(","); //$NON-NLS-1$

    // operators
    /** '+' token */
    static final public TokenType PLUS = new TokenType("+"); //$NON-NLS-1$

    /** '-' token */
    static final public TokenType MINUS = new TokenType("-"); //$NON-NLS-1$

    /** '*' token */
    static final public TokenType STAR = new TokenType("*"); //$NON-NLS-1$

    /** '/' token */
    static final public TokenType SLASH = new TokenType("/"); //$NON-NLS-1$

    /** '%' token */
    static final public TokenType MOD = new TokenType("%"); //$NON-NLS-1$

    /** '!' token */
    static final public TokenType EXCLAMATION = new TokenType("!"); //$NON-NLS-1$

    /** token signifying end of input */
    static final public TokenType EOI = new TokenType("end of input"); //$NON-NLS-1$

    private TokenType(String v) {
        tokenValue = v;
    }

    /**
     * @see Object#toString()
     */
    public String toString() {
        return "'" + tokenValue + "'"; //$NON-NLS-1$ //$NON-NLS-2$
    }
}