/*******************************************************************************
 * Copyright (c) 2004, 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Dec 27, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 * 
 * $Id: OBPreferencePage.java,v 1.1 2006/01/23 16:08:26 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FontFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import edu.wpi.ebob.ObjBenchPlugin;

/**
 * This class represents the preference page for the Object Bench that is
 * contributed to the Preferences dialog.
 * <p>
 * This page is used to modify preferences only. They are stored in the
 * preference store that belongs to the main plug-in class. That way,
 * preferences can be accessed directly via the preference store.
 */
public class OBPreferencePage extends FieldEditorPreferencePage implements
        IWorkbenchPreferencePage {

    /**
     * The property for the font used in instances. The default for this is
     * whatever the default is on a particular operating system- no changes are
     * made by this plug-in.
     */
    public static final String P_FONT = "fontPref"; //$NON-NLS-1$

    /**
     * The property for whether the object bench views should use
     * full-qualified. The default is <code>false</code>.
     */
    public static final String P_FQ_CLASS_NAMES = "fqClassNamesPref"; //$NON-NLS-1$

    /**
     * The property for whether the system should ask for confirmation before
     * removing a selected instance. The default is <code>true</code>.
     */
    public static final String P_CONFIRM_REMOVE = "confirmRemovePref"; //$NON-NLS-1$

    /**
     * The property for the text color to be displayed in instances. The default
     * is <code>black</code> (or whatever the OS decides).
     */
    public static final String P_FG_COLOR = "fgColorPref"; //$NON-NLS-1$

    /**
     * The property for the background color to be displayed in instances. The
     * default is <code>129,121,229</code>.
     */
    public static final String P_BG_COLOR = "bgColorPref"; //$NON-NLS-1$

    /**
     * The property for the background color to be displayed in selected
     * instances. The default is <code>121,182,229</code>.
     */
    public static final String P_BG_COLOR_SELECTED = "selectedColorPref"; //$NON-NLS-1$
    
    /**
     * The property for whether the system should ask for confirmation before
     * initializing the Object Bench. The default is <code>true</code>.
     */
    public static final String P_CONFIRM_INITIALIZE = "confirmInitializePref"; //$NON-NLS-1$

    public OBPreferencePage() {
        super(GRID);
        setPreferenceStore(ObjBenchPlugin.getDefault().getPreferenceStore());
        setDescription(ObjBenchPlugin
                .getResourceString("OBPreferencePage.description")); //$NON-NLS-1$
    }

    /**
     * Creates the field editors. Field editors are abstractions of the common
     * GUI blocks needed to manipulate various types of preferences. Each field
     * editor knows how to save and restore itself.
     */
    public void createFieldEditors() {
        addField(new BooleanFieldEditor(
                P_FQ_CLASS_NAMES,
                ObjBenchPlugin
                        .getResourceString("OBPreferencePage.fqNamesLabel"), getFieldEditorParent())); //$NON-NLS-1$
        addField(new BooleanFieldEditor(
                P_CONFIRM_REMOVE,
                ObjBenchPlugin
                        .getResourceString("OBPreferencePage.confirmRemoveLabel"), getFieldEditorParent())); //$NON-NLS-1$
        addField(new BooleanFieldEditor(
                P_CONFIRM_INITIALIZE,
                ObjBenchPlugin
                        .getResourceString("OBPreferencePage.confirmInitializeLabel"), getFieldEditorParent())); //$NON-NLS-1$
        addField(new FontFieldEditor(P_FONT, ObjBenchPlugin
                .getResourceString("OBPreferencePage.fontChoiceLabel"), //$NON-NLS-1$
                getFieldEditorParent()));
        addField(new ColorFieldEditor(P_FG_COLOR, ObjBenchPlugin
                .getResourceString("OBPreferencePage.fgColorChoiceLabel"), //$NON-NLS-1$
                getFieldEditorParent()));
        addField(new ColorFieldEditor(P_BG_COLOR, ObjBenchPlugin
                .getResourceString("OBPreferencePage.bgColorChoiceLabel"), //$NON-NLS-1$
                getFieldEditorParent()));
        addField(new ColorFieldEditor(
                P_BG_COLOR_SELECTED,
                ObjBenchPlugin
                        .getResourceString("OBPreferencePage.bgSelectedColorChoiceLabel"), //$NON-NLS-1$
                getFieldEditorParent()));
    }

    public void init(IWorkbench workbench) {
        // nothing to do, really
    }
}