/*******************************************************************************
 * Copyright (c) 2004, 2005 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 4, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *	  Justin Braga - catch IllegalStateException from getClass
 * 
 * $Id: ObjBenchView.java,v 1.1 2006/01/23 16:08:08 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.views;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.ViewPart;

import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.actions.ClearObjectBenchAction;
import edu.wpi.ebob.actions.CreateInstanceAction;
import edu.wpi.ebob.actions.RemoveInstanceAction;
import edu.wpi.ebob.internal.gui.InstanceContainer;
import edu.wpi.ebob.model.cm.ClassManagerEvent;
import edu.wpi.ebob.model.cm.IClassManagerListener;

/**
 * The main view for the Object Bench.
 * 
 * @author <a href="mailto:imotic@users.sourceforge.net">Liam Morley </a>
 */
public class ObjBenchView extends ViewPart {
    public ObjBenchView() {
        // change the name and content description if the class manager gets
        // dirty
        cmListener = new IClassManagerListener() {
            // the original partName and contentDescription
            private String partName, contentDescription;

            public void dirtyFlagSet(ClassManagerEvent event) {
                if (event.getNewFlag()) {
                    if (!event.getOldFlag()) {
                        partName = getPartName();
                        contentDescription = getContentDescription();
                        setPartName('*' + partName);
                        setContentDescription(ObjBenchPlugin
                                .getResourceString("ObjBenchView.dirtyContentDescription")); //$NON-NLS-1$
                    }
                } else if (event.getOldFlag()) {
                    setPartName(partName);
                    setContentDescription(contentDescription);
                }
            }
        };
        ObjBenchPlugin.getDefault().getClassManager().addClassManagerListener(
                cmListener);
    }

    /** the container for the gui objects */
    private InstanceContainer container;

    /** action for adding a new instance */
    private Action addInstanceAction;

    /** action for removing an instance */
    private RemoveInstanceAction removeInstanceAction;

    private IClassManagerListener cmListener;

    private Action clearBenchAction;

    private void contributeToActionBars() {
        IActionBars bars = getViewSite().getActionBars();
        fillLocalPullDown(bars.getMenuManager());
        fillLocalToolBar(bars.getToolBarManager());
    }

    /**
     * This is a callback that will allow us to create the viewer and initialize
     * it.
     */
    public void createPartControl(Composite parent) {
        container = new InstanceContainer(parent);
        ObjBenchPlugin.getDefault().getBench()
                .addObjectBenchListener(container);
        getSite().setSelectionProvider(container);

        makeActions();
        hookContextMenu();
        contributeToActionBars();
    }

    private void fillLocalPullDown(IMenuManager manager) {
        manager.add(addInstanceAction);
        manager.add(removeInstanceAction);
        manager.add(clearBenchAction);
    }

    private void fillLocalToolBar(IToolBarManager manager) {
        manager.add(addInstanceAction);
        manager.add(removeInstanceAction);
        manager.add(clearBenchAction);
    }

    private void hookContextMenu() {
        getSite().registerContextMenu(
                container.getContextMenu(new IAction[] { addInstanceAction }),
                getSite().getSelectionProvider());
    }

    private void makeActions() {
        addInstanceAction = new CreateInstanceAction(getSite().getShell());
        removeInstanceAction = new RemoveInstanceAction(container);
        clearBenchAction = new ClearObjectBenchAction();
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
        container.setFocus();
    }

    /**
     * Cleans things up properly.
     * 
     * @see org.eclipse.ui.IWorkbenchPart#dispose()
     */
    public void dispose() {
        removeInstanceAction.dispose();
        ObjBenchPlugin objBenchPlugin = ObjBenchPlugin.getDefault();
        objBenchPlugin.getBench().removeObjectBenchListener(container);
        objBenchPlugin.getClassManager().removeClassManagerListener(cmListener);
    }
}