/*******************************************************************************
 * Copyright (c) 2004 Worcester Polytechnic Institute.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Initially created on Nov 20, 2004
 *
 * Contributors:
 *    Liam Morley - initial API and implementation
 *    Justin Braga - Transition to EBOB Reflection
 * 	  Justin Braga - Catching CMExceptions and removed LabelProvider
 * 	  Justin Braga - Small Javadoc change
 *
 * $Id: AddToObjectBenchWizardPageExtension.java,v 1.1 2006/01/23 16:08:17 bigben Exp $
 *******************************************************************************/
package edu.wpi.ebob.wizards;

import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.*;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import edu.wpi.ebob.ObjBenchLogger;
import edu.wpi.ebob.ObjBenchPlugin;
import edu.wpi.ebob.internal.gui.dialogs.DetailsDialogUtil;
import edu.wpi.ebob.internal.wizards.ArgsCellModifier;
import edu.wpi.ebob.internal.wizards.ArgsContentProvider;
import edu.wpi.ebob.internal.wizards.ArgsLabelProvider;
import edu.wpi.ebob.internal.wizards.IArgsConstants;
import edu.wpi.ebob.model.Parameter;
import edu.wpi.ebob.model.cm.RemoteException;
import edu.wpi.ebob.model.cm.IClass;
import edu.wpi.ebob.model.cm.IConstructor;
import edu.wpi.ebob.model.cm.IObject;
import edu.wpi.ebob.model.parser.ParserException;

/**
 * Takes care of setting the constructor and the constructor arguments.
 */
public class AddToObjectBenchWizardPageExtension extends WizardPage {

    private ComboViewer constructorViewer;

    private Table table;

    private TableViewer tableViewer;

    /**
     * Constructor for SampleNewWizardPage.
     *  
     */
    public AddToObjectBenchWizardPageExtension() {
        super(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPageExtension.name")); //$NON-NLS-1$
        setTitle(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPageExtension.title")); //$NON-NLS-1$
        setDescription(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPageExtension.description")); //$NON-NLS-1$
    }

    /**
     * @param parent
     */
    private void createConstructorCombo(Composite parent) {
        Combo c = new Combo(parent, SWT.READ_ONLY);
        c.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        constructorViewer = new ComboViewer(c);
        constructorViewer.setContentProvider(new ArrayContentProvider());
        constructorViewer.setLabelProvider(new LabelProvider());
        constructorViewer
                .addSelectionChangedListener(new ISelectionChangedListener() {
                    public void selectionChanged(SelectionChangedEvent event) {
                        IConstructor con = (IConstructor) ((StructuredSelection) event
                                .getSelection()).getFirstElement();
                        IClass[] paramTypes;
                        try {
                            paramTypes = con.getParameterTypes();
                            tableViewer.setInput(getArgs(paramTypes));
                        } catch (RemoteException re) {
                            DetailsDialogUtil.openError(getShell(), re);
                            ObjBenchLogger.logError(re.getMessage(), re);
                        }
                        dialogChanged();
                    }
                });
    }

    /**
     * @see IDialogPage#createControl(Composite)
     */
    public void createControl(Composite parent) {
        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout());

        createConstructorCombo(container);
        createTable(container);
        createTableViewer();

        setControl(container);

        setMessage(ObjBenchPlugin
                .getResourceString("AddToObjectBenchWizardPageExtension.message")); //$NON-NLS-1$
        setPageComplete(false);
    }

    /**
     * Create the Table
     */
    private void createTable(Composite parent) {
        int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

        table = new Table(parent, style);
        table.setLayoutData(new GridData(GridData.FILL_BOTH));
        table.setLinesVisible(true);
        table.setHeaderVisible(true);

        new TableColumn(table, SWT.LEFT, 0)
                .setText(IArgsConstants.TABLE_COL_TYPE);
        new TableColumn(table, SWT.LEFT, 1)
                .setText(IArgsConstants.TABLE_COL_ARGUMENT);

        // adjust the column sizes the first time this is displayed
        table.addPaintListener(new PaintListener() {
            public void paintControl(PaintEvent e) {
                TableLayout tl = new TableLayout();
                tl.addColumnData(new ColumnWeightData(1));
                tl.addColumnData(new ColumnWeightData(1));
                table.setLayout(tl);
                table.layout();
                // only do this once
                table.removePaintListener(this);
            }
        });
    }

    /**
     * Create the TableViewer
     */
    private void createTableViewer() {
        tableViewer = new TableViewer(table);
        tableViewer.setColumnProperties(new String[] {
                IArgsConstants.TABLE_COL_TYPE,
                IArgsConstants.TABLE_COL_ARGUMENT });

        CellEditor[] editors = new CellEditor[2];
        editors[0] = new TextCellEditor(table);
        editors[1] = new TextCellEditor(table);
        // keep track of when the cell loses focus
        ((Text) editors[1].getControl()).addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                dialogChanged();
            }
        });

        tableViewer.setCellEditors(editors);
        ArgsCellModifier argsCellModifier = new ArgsCellModifier(tableViewer);
        tableViewer.setCellModifier(argsCellModifier);
        tableViewer.setContentProvider(new ArgsContentProvider());
        tableViewer.setLabelProvider(new ArgsLabelProvider());
    }

    /**
     * Ensures that both text fields are set.
     */
    private void dialogChanged() {
        if (!this.isCurrentPage())
            return;
        Parameter[] args = (Parameter[]) tableViewer.getInput();
        if (null != args) {
            boolean argsAreIncomplete = false;
            for (int i = 0; i < args.length; i++) {
                Parameter arg = args[i];
                try {
                    // don't update status if we have an incomplete argument
                    if (null == arg.getInput())
                        argsAreIncomplete = true;
                    else
                        arg.evaluate();
                } catch (ParserException pe) {
                    if (pe.getCause() instanceof RemoteException)
                        updateStatus(pe.getCause().getLocalizedMessage());
                    else
                        updateStatus(pe.getLocalizedMessage());
                    return;
                } catch (ClassCastException cce) {
                    updateStatus(cce.getLocalizedMessage());
                    return;
                } catch (Throwable t) {
                    t.printStackTrace();
                }
            }
            if (!argsAreIncomplete)
                updateStatus(null);
        }
    }

    /**
     * turns an array of classes into an array of argument pairs
     * 
     * @param classes
     *            the parameter types of a constructor
     * @return the given argument pairs for these classes
     * @see Parameter
     */
    private Parameter[] getArgs(IClass[] classes) {
        Parameter[] wrappers = new Parameter[classes.length];
        for (int i = 0; i < classes.length; i++)
            wrappers[i] = new Parameter(classes[i]);
        return wrappers;
    }

    public IObject[] getArguments() throws ParserException, ClassCastException {
        Parameter[] args = (Parameter[]) tableViewer.getInput();
        IObject[] returnedArgs = new IObject[args.length];
        for (int i = 0; i < args.length; i++)
            returnedArgs[i] = args[i].evaluate();
        return returnedArgs;
    }

    public IConstructor getConstructor() {
        return (IConstructor) ((StructuredSelection) constructorViewer
                .getSelection()).getFirstElement();
    }

    /**
     * sets the instance class (which in turn determines which constructors will
     * be visible). Called by the wizard when this page becomes visible.
     * 
     * @param c
     */
    void setInstanceClass(IClass c) {
        try {
            constructorViewer.setInput(c.getConstructors());
            tableViewer.setInput(null);
        } catch (RemoteException cme) {
            MessageDialog.openError(getShell(), ObjBenchPlugin
                    .getResourceString("Dialog.title"), cme //$NON-NLS-1$
                    .getLocalizedMessage());
            ObjBenchLogger.logError(cme.getMessage(), cme);
        }
    }

    private void updateStatus(String message) {
        setErrorMessage(message);
        setPageComplete(message == null);
        if (message == null)
            setMessage(ObjBenchPlugin
                    .getResourceString("AddToObjectBenchWizardPageExtension.finish.message")); //$NON-NLS-1$
    }
}